==> openmsx_0.7.0.orig.tar.gz: openmsx-0.7.0/src/video/SDLSnow.cc <==
// $Id: SDLSnow.cc 8223 2008-09-14 19:17:24Z m9710797 $

#include "SDLSnow.hh"
#include "OutputSurface.hh"
#include "openmsx.hh"
#include "build-info.hh"
#include <cstring>

namespace openmsx {

template <class Pixel>
SDLSnow<Pixel>::SDLSnow(OutputSurface& output_)
	: Layer(COVER_FULL, Z_BACKGROUND)
	, output(output_)
{
	// Precalc gray values for noise
	for (unsigned i = 0; i < 256; ++i) {
		double t = i / 255.0;
		gray[i] = output.mapRGB(t, t, t);
	}
}

// random routine, less random than libc rand(), but a lot faster
static int random()
{
	static int seed = 1;

	const int IA = 16807;
	const int IM = 2147483647;
	const int IQ = 127773;
	const int IR = 2836;

	int k = seed / IQ;
	seed = IA * (seed - k * IQ) - IR * k;
	if (seed < 0) seed += IM;
	return seed;
}

template <class Pixel>
void SDLSnow<Pixel>::paint()
{
	output.lock();
	const unsigned width = output.getWidth();
	const unsigned height = output.getHeight();
	for (unsigned y = 0; y < height; y += 2) {
		Pixel* p0 = output.getLinePtrDirect<Pixel>(y + 0);
		Pixel* p1 = output.getLinePtrDirect<Pixel>(y + 1);
		for (unsigned x = 0; x < width; x += 2) {
			byte a = byte(random());
			p0[x + 0] = p0[x + 1] = gray[a];
		}
		memcpy(p1, p0, width * sizeof(Pixel));
	}

	// TODO: Mark dirty in 100ms.
}

template <class Pixel>
const std::string& SDLSnow<Pixel>::getName()
{
	static const std::string NAME = "SDLSnow";
	return NAME;
}

// Force template instantiation.
#if HAVE_16BPP
template class SDLSnow<word>;
#endif
#if HAVE_32BPP
template class SDLSnow<unsigned>;
#endif

} // namespace openmsx

==> openmsx_0.8.0.orig.tar.gz: openmsx-0.8.0/src/video/SDLSnow.cc <==
// $Id: SDLSnow.cc 9404 2009-03-20 21:34:28Z m9710797 $

#include "SDLSnow.hh"
#include "OutputSurface.hh"
#include "Display.hh"
#include "openmsx.hh"
#include "build-info.hh"
#include <cstring>

namespace openmsx {

template <class Pixel>
SDLSnow<Pixel>::SDLSnow(OutputSurface& output, Display& display_)
	: Layer(COVER_FULL, Z_BACKGROUND)
	, display(display_)
{
	// Precalc gray values for noise
	for (unsigned i = 0; i < 256; ++i) {
		double t = i / 255.0;
		gray[i] = output.mapRGB(t, t, t);
	}
}

// random routine, less random than libc rand(), but a lot faster
static int random()
{
	static int seed = 1;

	const int IA = 16807;
	const int IM = 2147483647;
	const int IQ = 127773;
	const int IR = 2836;

	int k = seed / IQ;
	seed = IA * (seed - k * IQ) - IR * k;
	if (seed < 0) seed += IM;
	return seed;
}

template <class Pixel>
void SDLSnow<Pixel>::paint(OutputSurface& output)
{
	output.lock();
	const unsigned width = output.getWidth();
	const unsigned height = output.getHeight();
	for (unsigned y = 0; y < height; y += 2) {
		Pixel* p0 = output.getLinePtrDirect<Pixel>(y + 0);
		Pixel* p1 = output.getLinePtrDirect<Pixel>(y + 1);
		for (unsigned x = 0; x < width; x += 2) {
			byte a = byte(random());
			p0[x + 0] = p0[x + 1] = gray[a];
		}
		memcpy(p1, p0, width * sizeof(Pixel));
	}

	display.repaintDelayed(100 * 1000); // 10fps
}

template <class Pixel>
const std::string& SDLSnow<Pixel>::getName()
{
	static const std::string NAME = "SDLSnow";
	return NAME;
}

// Force template instantiation.
#if HAVE_16BPP
template class SDLSnow<word>;
#endif
#if HAVE_32BPP
template class SDLSnow<unsigned>;
#endif

} // namespace openmsx
